@@ -1,13 +1,13 @@
 #
-# $Id: Metabrik.pm,v 1418b1e3b32f 2016/01/02 17:46:47 gomor $
+# $Id: Metabrik.pm,v 54787ab9963e 2016/02/18 17:46:41 gomor $
 #
 package Metabrik;
 use strict;
 use warnings;
 
 # Breaking.Feature.Fix
-our $VERSION = '1.20';
-our $FIX = '0';
+our $VERSION = '1.21';
+our $FIX = '1';
 
 use base qw(Class::Gomor::Hash);
 
@@ -50,7 +50,7 @@ sub brik_license {
 
 sub brik_properties {
    return {
-      revision => '$Revision: 1418b1e3b32f $',
+      revision => '$Revision: 54787ab9963e $',
       author => 'GomoR <GomoR[at]metabrik.org>',
       license => 'http://opensource.org/licenses/BSD-3-Clause',
       tags => [ ],
@@ -98,6 +98,7 @@ sub brik_properties {
          brik_check_require_modules => [ ],
          brik_check_require_binaries => [ ],
          brik_check_properties => [ ],
+         brik_check_use_properties => [ ],
          brik_checks => [ ],
          brik_has_binary => [ qw(binary) ],
          brik_has_module => [ qw(module) ],
@@ -138,7 +139,7 @@ sub brik_help_set {
       my $attributes = $class->brik_attributes;
 
       if (exists($attributes->{$attribute})) {
-         my $help = sprintf("set %s %s ", $name, $attribute);
+         my $help = sprintf("%s ", $attribute);
          for (@{$attributes->{$attribute}}) {
             $help .= "<$_> ";
          }
@@ -165,9 +166,15 @@ sub brik_help_run {
       my $commands = $class->brik_commands;
 
       if (exists($commands->{$command})) {
-         my $help = sprintf("run %s %s ", $name, $command);
+         my $help = sprintf("%s ", $command);
          for (@{$commands->{$command}}) {
-            $help .= "<$_> ";
+            if (m{\|OPTIONAL}) {
+               s/\|OPTIONAL\s*$//;
+               $help .= "[ <$_> ] ";
+            }
+            else {
+               $help .= "<$_> ";
+            }
          }
          return $help;
       }
@@ -176,6 +183,18 @@ sub brik_help_run {
    return;
 }
 
+sub _msg {
+   my $self = shift;
+   my ($class, $msg) = @_;
+
+   $msg ||= 'undef';
+
+   $class = lc($class);
+   $class =~ s/^metabrik:://i;
+
+   return lc($class).": $msg";
+}
+
 sub _log_info {
    my $self = shift;
    my ($msg) = @_;
@@ -204,7 +223,8 @@ sub _log_error {
       return $self->log->error($msg, $class);
    }
    else {
-      print("[-] $class: $msg\n");
+      my $str = $self->_msg($class, $msg);
+      print "[-] $str\n";
    }
 
    return;
@@ -222,7 +242,8 @@ sub _log_fatal {
       return $self->log->fatal($msg, $class);
    }
    else {
-      die("[F] $class: $msg\n");
+      my $str = $self->_msg($class, $msg);
+      die("[F] $str\n");
    }
 
    return;
@@ -240,7 +261,8 @@ sub _log_warning {
       return $self->log->warning($msg, $class);
    }
    else {
-      print("[!] $class: $msg\n");
+      my $str = $self->_msg($class, $msg);
+      print("[!] $str\n");
    }
 
    return 1;
@@ -258,7 +280,8 @@ sub _log_verbose {
       return $self->log->verbose($msg, $class);
    }
    else {
-      print("[*] $class: $msg\n");
+      my $str = $self->_msg($class, $msg);
+      print("[*] $str\n");
    }
 
    return 1;
@@ -280,7 +303,8 @@ sub _log_debug {
       return $self->log->debug($msg, $class);
    }
    else {
-      print("[D] $class: $msg\n");
+      my $str = $self->_msg($class, $msg);
+      print("[D] $str\n");
    }
 
    return 1;
@@ -288,7 +312,7 @@ sub _log_debug {
 
 sub brik_check_properties {
    my $self = shift;
-   my ($properties, $use_properties) = @_;
+   my ($properties) = @_;
 
    my $name = $self->brik_name;
    if (! $self->can('brik_properties')) {
@@ -296,7 +320,6 @@ sub brik_check_properties {
    }
 
    $properties ||= $self->brik_properties;
-   $use_properties ||= $self->brik_use_properties;
 
    my $error = 0;
 
@@ -341,20 +364,6 @@ sub brik_check_properties {
          $error++;
       }
    }
-   for my $key (keys %$use_properties) {
-      if (! exists($valid_keys{$key})) {
-         print("[-] brik_check_properties: brik_use_properties has invalid key [$key]\n");
-         $error++;
-      }
-      elsif ($key eq 'tags' && ref($use_properties->{$key}) ne 'ARRAY') {
-         print("[-] brik_check_properties: brik_use_properties with key [$key] is not an ARRAYREF\n");
-         $error++;
-      }
-      elsif ($key ne 'revision' && $key ne 'author' && $key ne 'license' && $key ne 'tags' && ref($use_properties->{$key}) ne 'HASH') {
-         print("[-] brik_check_properties: brik_use_properties with key [$key] is not a HASHREF\n");
-         $error++;
-      }
-   }
 
    # Check HASHREFs contains pointers to ARRAYREFs
    for my $key (keys %$properties) {
@@ -367,16 +376,6 @@ sub brik_check_properties {
          }
       }
    }
-   for my $key (keys %$use_properties) {
-      next if ($key eq 'revision' || $key eq 'author' || $key eq 'license' || $key eq 'tags' || $key eq 'attributes_default');
-
-      for my $subkey (keys %{$use_properties->{$key}}) {
-         if (ref($use_properties->{$key}->{$subkey}) ne 'ARRAY') {
-            print("[-] brik_check_properties: brik_use_properties with key [$key] and subkey [$subkey] is not an ARRAYREF\n");
-            $error++;
-         }
-      }
-   }
 
    if ($error) {
       print("[-] brik_check_properties: Brik [$name] has invalid properties ($error error(s) found)\n");
@@ -463,17 +462,10 @@ sub brik_check_use_properties {
 sub brik_checks {
    my $self = shift;
 
-   my $r = $self->brik_check_properties;
-   return unless $r;
-
-   $r = $self->brik_check_use_properties;
-   return unless $r;
-
-   $r = $self->brik_check_require_modules;
-   return unless $r;
-
-   $r = $self->brik_check_require_binaries;
-   return unless $r;
+   $self->brik_check_properties or return;
+   $self->brik_check_use_properties or return;
+   $self->brik_check_require_modules or return;
+   $self->brik_check_require_binaries or return;
 
    return $self;
 }
@@ -484,12 +476,9 @@ sub new {
    );
 
    my $r = $self->brik_create_attributes;
-   return unless $r;
-
-   $r = $self->brik_set_default_attributes;
-   return unless $r;
-
-   $self->brik_checks or return;
+   if (! defined($r)) {
+      return $self->_log_error("new: brik_create_attributes failed");
+   }
 
    return $self->brik_preinit;
 }
@@ -500,10 +489,9 @@ sub new_no_checks {
    );
 
    my $r = $self->brik_create_attributes;
-   return unless $r;
-
-   $r = $self->brik_set_default_attributes;
-   return unless $r;
+   if (! defined($r)) {
+      return $self->_log_error("new_no_checks: brik_create_attributes failed");
+   }
 
    return $self->brik_preinit;
 }
@@ -521,6 +509,19 @@ sub new_from_brik {
    my $con = $brik->context;
    my $she = $brik->shell;
 
+   if (! defined($log)) {
+      return $self->_log_error("new_from_brik: log Attribute is undef");
+   }
+   if (! defined($glo)) {
+      return $self->_log_error("new_from_brik: glo Attribute is undef");
+   }
+   if (! defined($con)) {
+      return $self->_log_error("new_from_brik: con Attribute is undef");
+   }
+   if (! defined($she)) {
+      return $self->_log_error("new_from_brik: she Attribute is undef");
+   }
+
    return $self->new(
       log => $log,
       global => $glo,
@@ -542,6 +543,19 @@ sub new_from_brik_no_checks {
    my $con = $brik->context;
    my $she = $brik->shell;
 
+   if (! defined($log)) {
+      return $self->_log_error("new_from_brik_no_checks: log Attribute is undef");
+   }
+   if (! defined($glo)) {
+      return $self->_log_error("new_from_brik_no_checks: glo Attribute is undef");
+   }
+   if (! defined($con)) {
+      return $self->_log_error("new_from_brik_no_checks: con Attribute is undef");
+   }
+   if (! defined($she)) {
+      return $self->_log_error("new_from_brik_no_checks: she Attribute is undef");
+   }
+
    return $self->new_no_checks(
       log => $log,
       global => $glo,
@@ -622,6 +636,45 @@ sub brik_set_default_attributes {
       }
    }
 
+   # Special case: automatic setting of some defaults (datadir)
+   # No inheritance here, it is just for currently instanciated Brik.
+   my $global = $self->global;
+   if (defined($global)
+   &&  exists($self->brik_properties->{attributes})
+   &&  exists($self->brik_properties->{attributes}->{datadir})) {
+      my $datadir = $self->datadir;
+
+      my $dir;
+      # If datadir is set by user, we use it blindly.
+      # Usually, only core::global will have it set.
+      if (defined($datadir)) {
+         $dir = $datadir;
+      }
+      # Else, we build it.
+      else {
+         my $global_datadir = $self->global->datadir;
+         $dir = $global_datadir;
+
+         (my $subdir = $self->brik_name) =~ s/::/-/g;
+         if (length($subdir)) {
+            $dir .= '/'.$subdir;
+         }
+
+         $self->datadir($dir);
+      }
+
+      if (! -d $dir) {
+         mkdir($dir)
+            or return $self->_log_error("brik_set_default_attributes: mkdir [$dir] failed: $!");
+      }
+   }
+
+   return 1;
+}
+
+sub brik_set_use_default_attributes {
+   my $self = shift;
+
    # Set default Attributes from brik_use_properties, no hierarchy, just inheritance
    my $class = $self->brik_class;
    if ($self->can('brik_use_properties') && exists($self->brik_use_properties->{attributes_default})) {
@@ -715,7 +768,7 @@ sub brik_check_require_binaries {
    my $error = 0;
    for my $binary (keys %binaries_found) {
       if (! $binaries_found{$binary}) {
-         $self->_log_error("brik_check_require_binaries: binary [$binary] not found in \$PATH");
+         $self->_log_error("brik_check_require_binaries: binary [$binary] not found in PATH");
          $error++;
       }
    }
@@ -1075,32 +1128,24 @@ sub brik_has_binary {
 sub brik_preinit {
    my $self = shift;
 
-   if ($self->brik_has_attribute('datadir')) {
-      my $datadir = $self->datadir;
+   my $r = $self->brik_set_default_attributes;
+   if (! defined($r)) {
+      return $self->_log_error("brik_preinit: brik_set_default_attributes failed");
+   }
 
-      my $dir;
-      # If datadir is set by user, we use it blindly.
-      # Usually, only core::global will have it set.
-      if (defined($datadir)) {
-         $dir = $datadir;
-      }
-      # Else, we build it.
-      else {
-         my $global_datadir = $self->global->datadir;
-         $dir = $global_datadir;
+   # We have to put it here, cause brik_use_properties method is called, and 
+   # we want some default attributes to be set defore that (datadir special case)
+   # brik_preinit method is called by new(), so no problem, it will be checked.
+   $r = $self->brik_checks;
+   if (! defined($r)) {
+      return $self->_log_error("brik_preinit: brik_checks failed");
+   }
 
-         (my $subdir = $self->brik_name) =~ s/::/-/g;
-         if (length($subdir)) {
-            $dir .= '/'.$subdir;
-         }
-
-         $self->datadir($dir);
-      }
-
-      if (! -d $dir) {
-         mkdir($dir)
-            or return $self->_log_error("brik_preinit: mkdir [$dir] failed: $!");
-      }
+   # Now, we can set default Attributes from brik_use_properties, all brik_properties
+   # Attributes should be inited with defaults.
+   $r = $self->brik_set_use_default_attributes;
+   if (! defined($r)) {
+      return $self->_log_error("brik_preinit: brik_set_use_default_attributes failed");
    }
 
    return $self;
@@ -1182,6 +1227,10 @@ sub brik_help_run_empty_array_arg {
    my $brik = lc($package);
    $brik =~ s/^metabrik:://;
 
+   if (ref($argument) ne 'ARRAY') {
+      return $self->log->error("$brik: $command: Argument [$argument] is not an ARRAY");
+   }
+
    if (@$argument <= 0) {
       return $self->log->error("$brik: $command: ARRAY Argument [$argument] is empty");
    }
@@ -1364,6 +1413,8 @@ L<help core::global>
 
 =item B<brik_set_default_attributes>
 
+=item B<brik_set_use_default_attributes>
+
 =item B<brik_fini>
 
 =back
