@@ -1,19 +1,19 @@
 #
-# $Id: Context.pm,v 1418b1e3b32f 2016/01/02 17:46:47 gomor $
+# $Id: Context.pm,v e86617fa210d 2016/02/14 17:56:59 gomor $
 #
 package Metabrik::Core::Context;
 use strict;
 use warnings;
 
 # Breaking.Feature.Fix
-our $VERSION = '1.20';
-our $FIX = '0';
+our $VERSION = '1.21';
+our $FIX = '1';
 
 use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 1418b1e3b32f $',
+      revision => '$Revision: e86617fa210d $',
       tags => [ qw(main core) ],
       attributes => {
          _lp => [ qw(INTERNAL) ],
@@ -737,6 +737,11 @@ sub run {
       return $self->log->error($self->brik_help_run('run'));
    }
 
+   if ($self->debug) {
+      my ($module, $file, $line) = caller();
+      $self->log->debug("run: called by module [$module] from [$file] line[$line]");
+   }
+
    my $r = $self->call(sub {
       my %args = @_;
 
