@@ -1,5 +1,5 @@
 #
-# $Id: Log.pm,v 1418b1e3b32f 2016/01/02 17:46:47 gomor $
+# $Id: Log.pm,v 54787ab9963e 2016/02/18 17:46:41 gomor $
 #
 # core::log Brik
 #
@@ -8,14 +8,14 @@ use strict;
 use warnings;
 
 # Breaking.Feature.Fix
-our $VERSION = '1.20';
-our $FIX = '0';
+our $VERSION = '1.21';
+our $FIX = '1';
 
 use base qw(Metabrik);
 
 sub brik_properties {
    return {
-      revision => '$Revision: 1418b1e3b32f $',
+      revision => '$Revision: 54787ab9963e $',
       tags => [ qw(main core) ],
       attributes => {
          color => [ qw(0|1) ],
@@ -39,6 +39,42 @@ sub brik_properties {
    };
 }
 
+sub brik_preinit {
+   my $self = shift;
+
+   # We will do a brik_init here, so we have to force the brik_preinit before.
+   $self->SUPER::brik_preinit(@_) or return;
+
+   my $context = $self->context;
+
+   # We replace the current logging Brik by this one,
+   # but only after core::context has been created and initialized.
+   if (defined($context)) {
+      $context->{log} = $self;
+      for my $this (keys %{$context->used}) {
+         $context->{used}->{$this}->{log} = $self;
+      }
+
+      # We have to init this new log Brik, because previous one
+      # was already inited at this stage. We have to keep the same init context.
+      $self->brik_init or return $self->log->error("brik_preinit: brik_init error");
+   }
+
+   return $self;
+}
+
+sub brik_init {
+   my $self = shift;
+
+   # Makes STDOUT file handle unbuffered
+   my $current = select;
+   select(STDOUT);
+   $|++;
+   select($current);
+
+   return $self->SUPER::brik_init(@_);
+}
+
 sub _msg {
    my $self = shift;
    my ($brik, $msg) = @_;
@@ -203,6 +239,10 @@ L<help core::log>
 
 =over 4
 
+=item B<brik_preinit>
+
+=item B<brik_init>
+
 =item B<brik_properties>
 
 =item B<debug>
