@@ -1,5 +1,5 @@
 #
-# $Id: Shell.pm,v 1418b1e3b32f 2016/01/02 17:46:47 gomor $
+# $Id: Shell.pm,v e86617fa210d 2016/02/14 17:56:59 gomor $
 #
 # core::shell Brik
 #
@@ -8,8 +8,8 @@ use strict;
 use warnings;
 
 # Breaking.Feature.Fix
-our $VERSION = '1.20';
-our $FIX = '0';
+our $VERSION = '1.21';
+our $FIX = '1';
 
 use base qw(Term::Shell Metabrik);
 
@@ -17,7 +17,7 @@ use IO::All;
 
 sub brik_properties {
    return {
-      revision => '$Revision: 1418b1e3b32f $',
+      revision => '$Revision: e86617fa210d $',
       tags => [ qw(main core) ],
       attributes => {
          echo => [ qw(0|1) ],
@@ -43,6 +43,7 @@ sub brik_properties {
          show_inherited_all => [ qw(0|1) ],
          show_all => [ qw(0|1) ],  # Both Attributes and Commands for base and inherited
          aliases_completion => [ qw(0|1) ], # Complete aliases to show original Command
+         ps1 => [ qw(prompt) ],
          # These are used by Term::Shell
          #path_home => [ qw(directory) ],
          #path_cwd => [ qw(directory) ],
@@ -73,7 +74,7 @@ sub brik_properties {
          show_inherited_commands => 0,
          show_inherited_all => 0,
          show_all => 0,
-         aliases_completion => 1,
+         aliases_completion => 0,
       },
       commands => {
          splash => [ ],
@@ -305,24 +306,25 @@ sub rl_complete {
 
       $self->debug && $self->log->debug("rl_complete: comp[@comp]");
 
+      # XXX: broken feature since Term::ReadLine::Gnu 1.27
       # If we found something and it's an alias, we complete with the original command
-      if (defined($comp[0]) && $self->comp_aliases && exists($self->{_aliases}{"run_".$comp[0]})) {
-         $self->debug && $self->log->debug("rl_complete: original[".$self->{_aliases}{"run_$comp[0]"}."]");
+      #if (defined($comp[0]) && $self->comp_aliases && exists($self->{_aliases}{"run_".$comp[0]})) {
+         #$self->debug && $self->log->debug("rl_complete: original[".$self->{_aliases}{"run_$comp[0]"}."]");
 
-         my @words = split(/\s+/, $self->{_aliases}{"run_$comp[0]"});
-         if (exists($self->{_aliases}{"run_$words[0]"})) {
-            @comp = $self->{_aliases}{"run_".$words[0]};
-         }
-         else {
-            @comp = $self->{_aliases}{"run_$comp[0]"};
-         }
-      }
-      # Or maybe it's a Brik, and we want to prefix it automagically with 'run' Command
-      elsif (my $available = $self->_word_may_be_brik($word)) {
-         for (keys %$available) {
-            push @comp, "run $_" if /^$word/;
-         }
-      }
+         #my @words = split(/\s+/, $self->{_aliases}{"run_$comp[0]"});
+         #if (exists($self->{_aliases}{"run_$words[0]"})) {
+            #@comp = $self->{_aliases}{"run_".$words[0]};
+         #}
+         #else {
+            #@comp = $self->{_aliases}{"run_$comp[0]"};
+         #}
+      #}
+      ## Or maybe it's a Brik, and we want to prefix it automagically with 'run' Command
+      #elsif (my $available = $self->_word_may_be_brik($word)) {
+         #for (keys %$available) {
+            #push @comp, "run $_" if /^$word/;
+         #}
+      #}
    }
    # If it's a subcommand, send it to any custom completion function for the
    # function:
@@ -332,7 +334,7 @@ sub rl_complete {
       @comp = $self->complete($command, $word, $line, $start);
    }
 
-   $self->debug && $self->log->debug("rl_complete: return comp[@comp]");
+   $self->debug && $self->log->debug("rl_complete: return comp[@comp] count[".scalar(@comp)."]");
 
    return @comp;
 }
@@ -356,7 +358,7 @@ sub _update_path_home {
    my $self = shift;
 
    #$self->path_home(_convert_path(home()));
-   $self->{path_home} = _convert_path(File::HomeDir->my_home);
+   $self->{path_home} = _convert_path(File::HomeDir->my_home || '/tmp');
 
    return 1;
 }
@@ -364,10 +366,10 @@ sub _update_path_home {
 sub _update_path_cwd {
    my $self = shift;
 
-   my $cwd = _convert_path(Cwd::getcwd());
+   my $cwd = _convert_path(Cwd::getcwd() || '/tmp');
    $self->debug && $self->log->debug("cwd [$cwd]");
    #my $home = $self->path_home;
-   my $home = $self->{path_home};
+   my $home = $self->{path_home} || '/tmp';
    $self->debug && $self->log->debug("home [$home]");
    $cwd =~ s/^$home/~/;
 
@@ -386,11 +388,12 @@ sub _update_prompt {
       $self->{prompt} = $prompt;
    }
    else {
+      my $ps1 = $self->ps1;
       #my $cwd = $self->path_cwd;
       my $cwd = $self->{path_cwd};
 
-      my $prompt = "Meta:$cwd> ";
-      #my $prompt = "Meta> ";
+      my $prompt = defined($ps1) ? "$ps1:$cwd> " : "Meta:$cwd> ";
+
       if ($^O =~ /win32/i) {
          $prompt =~ s/> /\$ /;
       }
@@ -971,12 +974,12 @@ sub run_help {
          my $commands = $self->get_help_commands($arg1);
 
          for my $attribute (sort { $a cmp $b } keys %$attributes) {
-            my $help = $used_brik->brik_help_set($attribute);
+            my $help = "set $arg1 ".$used_brik->brik_help_set($attribute);
             $self->log->info($help) if defined($help);
          }
 
          for my $command (sort { $a cmp $b } keys %$commands) {
-            my $help = $used_brik->brik_help_run($command);
+            my $help = "run $arg1 ".$used_brik->brik_help_run($command);
             $self->log->info($help) if defined($help);
          }
       }
@@ -1231,6 +1234,17 @@ sub run_run {
          die("interrupted by user\n");
       };
 
+      if ($self->debug) {
+         my ($module, $file, $line) = caller();
+         $self->log->debug("run_run: called by module [$module] from [$file] line[$line]");
+      }
+
+      # Hack for Term::Shell: we have to convert its exec function to our execute Command
+      my ($module) = caller();
+      if ($module eq 'Term::Shell' && $command eq 'exec') {
+         $command = 'execute';
+      }
+
       $r = $context->run($brik, $command, @args);
       if (! defined($r)) {
          return $self->log->error("run: unable to execute Command [$command] for Brik [$brik]");
@@ -1521,10 +1535,11 @@ sub catch_comp {
          }
       }
    }
+   # XXX: broken feature since Term::ReadLine::Gnu 1.27
    # If we just finished completing an alias, we return the full command
-   elsif (exists($self->{_aliases}{"run_$last"})) {
-      push @comp, $self->{_aliases}{"run_$last"};
-   }
+   #elsif (exists($self->{_aliases}{"run_$last"})) {
+      #push @comp, $self->{_aliases}{"run_$last"};
+   #}
    # Otherwise, we complete relative filenames
    else {
       my $path = '.';
